//--------------------------------------------------------------
// File     : stm32_ub_adc3_dma.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_ADC3_DMA_H
#define __STM32F4_UB_ADC3_DMA_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_adc.h"
#include "stm32f4xx_dma.h"


//--------------------------------------------------------------
// Liste aller ADC-Kanle
// (keine Nummer doppelt und von 0 beginnend)
//--------------------------------------------------------------
typedef enum {
  ADC_PC3 = 0,  // PC3
  ADC_PF6 = 1   // PF6
}ADC3d_NAME_t;

#define  ADC3d_ANZ   2 // Anzahl von ADC3d_NAME_t (maximum = 16)


//--------------------------------------------------------------
// Adressen der ADCs
// (siehe Seite 66+427 vom Referenz Manual)
//--------------------------------------------------------------
#define ADC_BASE_ADR        ((uint32_t)0x40012000)
#define ADC_ADC3_OFFSET     ((uint32_t)0x00000200)

//--------------------------------------------------------------
// Adressen der Register
// (siehe Seite 427+428 vom Referenz Manual)
//--------------------------------------------------------------
#define ADC_REG_DR_OFFSET         0x4C

#define ADC3_DR_ADDRESS    (ADC_BASE_ADR | ADC_ADC3_OFFSET | ADC_REG_DR_OFFSET)



//--------------------------------------------------------------
// ADC-Clock
// Max-ADC-Frq = 36MHz
// Grundfrequenz = APB2 (APB2=90MHz)
// Mgliche Vorteiler = 2,4,6,8
//--------------------------------------------------------------

//#define ADC3d_VORTEILER     ADC_Prescaler_Div2 // Frq = 45 MHz
#define ADC3d_VORTEILER     ADC_Prescaler_Div4   // Frq = 22.5 MHz
//#define ADC3d_VORTEILER     ADC_Prescaler_Div6 // Frq = 15 MHz
//#define ADC3d_VORTEILER     ADC_Prescaler_Div8 // Frq = 11.25 MHz


//--------------------------------------------------------------
// DMA Einstellung
// (siehe Seite 304+305 vom Referenz Manual)
// Moegliche DMAs fuer ADC3 :
//   DMA2_STREAM0_CHANNEL2
//   DMA2_STREAM1_CHANNEL2
//--------------------------------------------------------------

#define ADC3_DMA_STREAM0            0
//#define ADC3_DMA_STREAM1          1

#ifdef ADC3_DMA_STREAM0
 #define ADC3_DMA_STREAM            DMA2_Stream0
 #define ADC3_DMA_CHANNEL           DMA_Channel_2
#elif defined ADC3_DMA_STREAM1
 #define ADC3_DMA_STREAM            DMA2_Stream1
 #define ADC3_DMA_CHANNEL           DMA_Channel_2
#endif





//--------------------------------------------------------------
// Struktur eines ADC Kanals
//--------------------------------------------------------------
typedef struct {
  ADC3d_NAME_t ADC_NAME;  // Name
  GPIO_TypeDef* ADC_PORT; // Port
  const uint16_t ADC_PIN; // Pin
  const uint32_t ADC_CLK; // Clock
  const uint8_t ADC_CH;   // ADC-Kanal
}ADC3d_t;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void UB_ADC3_DMA_Init(void);
uint16_t UB_ADC3_DMA_Read(ADC3d_NAME_t adc_name);


//--------------------------------------------------------------
#endif // __STM32F4_UB_ADC3_DMA_H
